<?php
/**
 * Plugin Name: CustoMeow Product Personalizer
 * Version: 0.1.0
 * Author: CustoMeow
 * Author URI: https://customeow.io
 * Requires Plugins: woocommerce
 * Domain Path: /languages
 * description: Easy to use customization features, support for online 2D/3D preview, powerful and diverse components, enhance the custom shopping experience, improve conversion rates.
 * License: GPLv3
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 *
 * @package customeow-product-personalizer
 */

defined('ABSPATH') || exit;

if (!defined('CUSTOMEOW_PP_PLUGIN_FILE')) {
  define('CUSTOMEOW_PP_PLUGIN_FILE', __FILE__);
}

//load required classes
require_once 'includes/class-customeow-utilities.php';
require_once 'includes/class-customeow-request-log.php';
require_once 'includes/class-customeow-integration.php';
require_once 'includes/class-customeow-admin.php';
require_once 'includes/class-customeow-admin-dashboard.php';
require_once 'includes/class-customeow-template.php';

if (!class_exists('CustoMeow_PP_Base')) {
  /**
   * The CustoMeow_PP class.
   */
  class CustoMeow_PP_Base {

    /**
     * This class instance.
     *
     * @var \CustoMeow_PP_Base single instance of this class.
     */
    private static $instance;

    /**
     * Constructor.
     */
    public function __construct() {

      // WP REST API.
      $this->rest_api_init();

      // log
      CustoMeow_Request_Log::init();

      // admin
      if (is_admin()) {
        CustoMeow_Admin::init();
      }

      CustoMeow_Template::init();
    }

    /**
     * Cloning is forbidden.
     */
    public function __clone() {
      wc_doing_it_wrong(__FUNCTION__, __('Cloning is forbidden.', 'customeow-product-personalizer'), CustoMeow_Utilities::VERSION);
    }

    /**
     * Unserializing instances of this class is forbidden.
     */
    public function __wakeup() {
      wc_doing_it_wrong(__FUNCTION__, __('Unserializing instances of this class is forbidden.', 'customeow-product-personalizer'), CustoMeow_Utilities::VERSION);
    }

    /**
     * @description: 激活
     * @return {*}
     */
    public static function register_activation() {
      // 设置临时选项，用于激活后重定向
      set_transient('customeow_pp_plugin_activation', true, MINUTE_IN_SECONDS);

      // 发送激活请求
      self::set_edit_plugin_status(1);
    }

    /**
     * @description: 停用
     * @return {*}
     */
    public static function register_deactivation() {
      // 发送停用请求
      self::set_edit_plugin_status(3);
    }

    /**
     * @description: 卸载
     * @return {*}
     */
    public static function register_uninstall() {
      // 发送卸载请求
      self::set_edit_plugin_status(2);
    }

    /**
     * @description: 重定向到插件首页
     * @return {*}
     */
    public static function redirect_to_plugin_home() {
      // 检查是否存在临时选项，并确保不是网络激活
      if (get_transient('customeow_pp_plugin_activation')) {
        // 删除临时选项
        delete_transient('customeow_pp_plugin_activation');

        // 重定向到插件主页
        wp_safe_redirect(admin_url('admin.php?page=' . CustoMeow_Admin::MENU_SLUG_DASHBOARD));
        exit;
      }
    }

    /**
     * @description: 是否安装Customeow的插件:0|none=无,1|yes=已经安装已激活,2|no=未安装,3|inactive=已安装待激活
     * @param {*} $status
     * @return {*}
     */
    public static function set_edit_plugin_status($status = 0) {
      $go_url = CustoMeow_Utilities::get_customeow_go_host();
      $client = CustoMeow_Integration::instance()->get_client($go_url);
      try {
        $response = $client->post('/api/v1/store-platform/woocommerce/webhooks/plug-update', array(
          'storeURL' => get_home_url(),
          'pluginStatus' => $status,
        ));
        return $response;
      } catch (Exception $e) {
        return false;
      }
    }

    /**
     * Gets the main instance.
     *
     * Ensures only one instance can be loaded.
     *
     * @return \CustoMeow_PP_Base
     */
    public static function instance() {
      if (null === self::$instance) {
        self::$instance = new self();
      }

      return self::$instance;
    }

    /**
     * @return string
     */
    public static function get_asset_url() {
      return trailingslashit(plugin_dir_url(__FILE__)) . 'assets/';
    }

    private function rest_api_init() {
      if (!class_exists('WP_REST_Server')) {
        return;
      }

      add_action('rest_api_init', array($this, 'register_rest_routes'), 20);
    }

    /** 注册 rest api 路由 */
    public function register_rest_routes() {
      require_once 'includes/class-customeow-rest-api-controller.php';

      $MadeMinePDRestAPIController = new CustoMeow_PP_REST_API_Controller();
      $MadeMinePDRestAPIController->register_routes();
    }

  }
}

/**
 * 依赖于 woocommerce 插件
 */
function customeow_pp_missing_wc_notice() {
  /* translators: %s WC download URL link. */
  echo '<div class="error"><p><strong>' . sprintf(esc_html__('CustoMeow Product Personalizer requires WooCommerce to be installed and active. You can download %s here.', 'customeow-product-personalizer'), '<a href="https://woo.com/" target="_blank">WooCommerce</a>') . '</strong></p></div>';
}

function customeow_pp_init() {
  if (!class_exists('WooCommerce')) {
    add_action('admin_notices', 'customeow_pp_missing_wc_notice');
    return;
  }

  CustoMeow_PP_Base::instance();

  // 检查并执行重定向
  add_action('admin_init', array('CustoMeow_PP_Base', 'redirect_to_plugin_home'));
}
add_action('plugins_loaded', 'customeow_pp_init', 10);

// 注册插件激活钩子
register_activation_hook(__FILE__, array('CustoMeow_PP_Base', 'register_activation'));

// 注册插件停用钩子
register_deactivation_hook(__FILE__, array('CustoMeow_PP_Base', 'register_deactivation'));

// 注册插件卸载钩子
register_uninstall_hook(__FILE__, array('CustoMeow_PP_Base', 'register_uninstall'));
